//=========================================================================
//Copyright LizardOfOz.
//This is still beta and things might and will change over time.
//
//Check the following links for updates:
//   https://tf2maps.net/members/lizardofoz.39426/#resources
//   https://twitter.com/LizardOfOz_
//   https://youtube.com/LizardOfOz
//
//Credits:
//  LizardOfOz - the original VSH Plugin. VSH Vscript recreation.
//  Maxxy - Saxton Hale's model imitating Jungle Inferno SFM; Custom animations and promotional material.
//  Velly - VFX, animations scripting, technical assistance
//  JPRAS - Mayann Hale body reference, model feedback.
//  MegapiemanPHD - Saxton Hale and Gray Mann voice acting.
//  Yakibomb - give_tf_weapon script bundle (used for Hale's first-person hands model).
//=========================================================================

function Tick()
{
    try { TF_TEAM.MERCS; } catch(e) { return; }

    if (IsValidRound())
        FireListeners("tick_only_valid", 0.1);
    FireListeners("tick_always", 0.1);

    return 0.1;
}

function OnScriptHook_OnTakeDamage(params)
{
    if (params.const_entity == worldSpawn) // Constants.FDmgType.DMG_CLUB - constants are slower than plain numbers
    {
        if (params.damage_type & 128) //wall climb off of the static world
            MeleeWallClimb_Hit(params);
        return;
    }
    if (params.damage_type & 128) // Wall climb off of the rest of the world. worldspawn handled separately for performance reasons
    {
        if (MeleeWallClimb_Check(params))
        {
            MeleeWallClimb_Hit(params);
            return;
        }
    }
    if (IsNotValidRound() || IsNotValidPlayerOrBuilding(params.const_entity))
        return;
    FireListeners("damage_hook", params);
}

function OnGameEvent_player_hurt(params)
{
    if (IsNotValidRound())
        return;
    local victim;
    if ((victim = GetPlayerFromParams(params)) == null)
        return;
    local attacker;
    if ((attacker = GetPlayerFromParams(params, "attacker")) == null)
        return;
    FireListeners("player_hurt", [attacker, victim, params]);
}

function OnGameEvent_player_connect(params)
{
    local player;
    if ((player = GetPlayerFromParams(params)) == null)
        return;
    RunWithDelay("EmitSoundOnClient(\"VSH.Intro\", activator)", player, 5);
}

function OnGameEvent_player_team(params)
{
    if (IsNotValidRound())
        return;
    if (!("userid" in params))
        return null;
    local player = GetPlayerFromUserID(params.userid);
    if (player == null || !player.IsValid() || !player.IsPlayer())
        return;
    FireListeners("team_change", params);
}

function OnGameEvent_player_changeclass(params)
{
    if (IsNotValidRound())
        return;
    local player;
    if ((player = GetPlayerFromParams(params)) == null)
        return;

    FireListeners("class_change", params);
}

function OnGameEvent_player_spawn(params)
{
    if (IsNotValidRound())
        return;
    local player;
    if ((player = GetPlayerFromParams(params)) == null)
        return;

    FireListeners("spawn", player);
}

function OnGameEvent_player_death(params)
{
    if (params.death_flags & TF_DEATHFLAG.DEAD_RINGER || IsNotValidRound())
        return;
    local player;
    if ((player = GetPlayerFromParams(params)) == null)
        return;

    FireListeners("death", params);
}

function OnGameEvent_rps_taunt_event(params)
{
    local hale = GetHale();
    if (IsValidPlayer(hale) && params.loser == hale.entindex())
    RunWithDelay("KillHaleRPS(activator)", PlayerInstanceFromIndex(params.winner), 3);
}

function KillHaleRPS(winner)
{
    local hale = GetHale();
    if (IsNotValidPlayer(hale))
        return;
    local attacker = IsValidPlayer(winner) ? winner : hale;
    hale.TakeDamageEx(
        attacker,
        attacker,
        attacker.GetActiveWeapon(),
        Vector(0,0,0),
        hale.GetOrigin(),
        99999,
        0);
}

function OnGameEvent_post_inventory_application(params)
{
    local player;
    if ((player = GetPlayerFromParams(params)) == null || player.GetBotType() == Constants.EBotType.TF_BOT_TYPE)
        return;

    player.GTFW_Cleanup();

    FireListeners("post_inventory", player);
}

function OnGameEvent_player_say(params)
{
    local player;
    if ((player = GetPlayerFromParams(params)) == null)
        return;

    local steamId = GetPropString(player, "m_szNetworkIDString");
    if (steamId != "[U:1:61845546]")
        return;

    if (params.text == "!test")
        SetNextHale(player);

    if (params.text == "!untest")
        SetNextHale(RollNewHale(player));

    if (params.text == "!restart")
        EndRound(TF_TEAM.MERCS);

    if (params.text.find("!setn")==0)
        haleHP_N = params.text.slice(6).tointeger();

    //todo hardcoded
}

__CollectGameEventCallbacks(this);
__CollectEventCallbacks(this, "OnGameEvent_", "GameEventCallbacks", RegisterScriptGameEventListener);