//=========================================================================
//Copyright LizardOfOz.
//This is still beta and things might and will change over time.
//
//Check the following links for updates:
//   https://tf2maps.net/members/lizardofoz.39426/#resources
//   https://twitter.com/LizardOfOz_
//   https://youtube.com/LizardOfOz
//
//Credits:
//  LizardOfOz - the original VSH Plugin. VSH Vscript recreation.
//  Maxxy - Saxton Hale's model imitating Jungle Inferno SFM; Custom animations and promotional material.
//  Velly - VFX, animations scripting, technical assistance
//  JPRAS - Mayann Hale body reference, model feedback.
//  MegapiemanPHD - Saxton Hale and Gray Mann voice acting.
//  Yakibomb - give_tf_weapon script bundle (used for Hale's first-person hands model).
//=========================================================================

isRoundSetup <- true
isRoundOver <- false;
starterClass <- {}
isFeedbackRound <- false;
roundStartTime <- 0;

function StartNewRound()
{
    DoEntFire("game_text_victory_screen", "AddOutput", "message ", 0, null, null);
    Convars.SetValue("mp_bonusroundtime", GetPersistentVar("mp_bonusroundtime"));
    roundStartTime = Time();

    SetRoundSetup(true);
    isRoundOver = false;

    FireListeners("start", null);
}

function OnNewRoundStarted(void)
{
    if (isFeedbackRound || IsInWaitingForPlayers() || GetTotalPlayerCount() < 2)
        return;
    ResetRoundDamage();

    isRoundSetup = true;

    local hale = GetHale();
    if (hale != null)
    {
        local wearable = null;
        while (wearable = Entities.FindByClassname(wearable, "tf_wearable*"))
            if (wearable.GetOwner() == hale)
                wearable.Kill();
    }

    ProgressHaleQueue();

    local forceRespawn = Entities.FindByClassname(null, "game_forcerespawn");
    EntFireByHandle(forceRespawn, "ForceRespawn", "", 0, null, null);

    foreach (player in GetAllValidMercs())
    {
        player.SwitchTeam(TF_TEAM.MERCS);
        player.ForceRegenerateAndRespawn();
        player.KeyValueFromString("targetname", "");
        TrySpawnAsMerc(player);
    }

    haleConversionAttempts = 0;
    TrySpawnAsHale(GetHale());
    RunWithDelay("PlaySoundForAll(`VSH.Start`)", null, 3);
}
AddListener("start", OnNewRoundStarted, 1);

function SetRoundSetup(isSetup)
{
    //TrySpawnAsHale(GetHale());

    if (isSetup)
    {
        isRoundSetup = isSetup;
        return;
    }

    foreach (player in GetAllValidMercs())
    {
        starterClass[player] <- player.GetPlayerClass();
        if (IsMercDead(player))
            player.ForceRespawn();
    }
    TickSetupHealth(GetHale());
    isRoundSetup = isSetup;

    local respawnRoom = null;
    while (respawnRoom = Entities.FindByClassname(respawnRoom, "func_respawnroom"))
        respawnRoom.Kill();

    local gamerules = Entities.FindByClassname(null, "tf_gamerules");
    SetPropInt(gamerules, "m_iRoundState", 7);
}

function ProgressHaleQueue()
{
    local gamerules = Entities.FindByClassname(null, "tf_gamerules");
    if (GetPropInt(gamerules, "m_nHudType") != 2) //todo move to a better place
        SetPropInt(gamerules, "m_nHudType", 2);
    gamerules.ValidateScriptScope();
    local globalVars = gamerules.GetScriptScope();

    if ("nextHale" in globalVars && IsValidPlayer(globalVars.nextHale))
    {
        SetHale(globalVars.nextHale);
        globalVars.lastHale <- GetHale();
    }
    else
    {
        local newHale = RollNewHale("lastHale" in globalVars ? globalVars.lastHale : null);
        globalVars.lastHale <- newHale;
        SetHale(newHale);
    }
    globalVars.nextHale <- null;
}

function RollNewHale(ignoredPlayer)
{
    local players = [];
    foreach (player in GetAllValidPlayers())
        if (player != ignoredPlayer)
            players.push(player);

    if (players.len() > 0)
        return players[RandomInt(0, players.len() - 1)];

    local theOnlyPlayer = Entities.FindByClassname(null, "player");
    return (theOnlyPlayer != null && !theOnlyPlayer.IsFakeClient()) ? theOnlyPlayer : null ;
    //return (theOnlyPlayer != null) ? theOnlyPlayer : null;
}

function HasRoundEnded(interval)
{
    if (IsInWaitingForPlayers() || GetTotalPlayerCount() < 2)
        return;
    if (GetAliveMercCount() <= 0)
        EndRound(TF_TEAM.HALE);
    local hale = GetHale();
    if (!IsRoundSetup() && (IsNotValidPlayer(hale) || IsPlayerDead(hale)))
        EndRound(TF_TEAM.MERCS);
}
AddListener("tick_always", HasRoundEnded, 1);

function EndRound(winner)
{
    if (isRoundOver)
        return;
    PlaySoundForAll(winner == TF_TEAM.HALE ? "VSH.HaleWins" : "VSH.HaleLoses");

    local roundWin = Entities.FindByClassname(null, "game_round_win");
    if (roundWin == null)
    {
        roundWin = SpawnEntityFromTable("game_round_win",
        {
            win_reason = "0",
            force_map_reset = "1", //not having
            TeamNum = "0",         //these 3 lines
            switch_teams = "0"     //causes the crash when trying to fire game_round_win
        });
    }
    EntFireByHandle(roundWin, "SetTeam", "" + winner, 0, null, null);
    EntFireByHandle(roundWin, "RoundWin", "", 0, null, null);
    PlaySoundForAll(winner == TF_TEAM.HALE ? "VSH.HaleWins" : "VSH.HaleLoses");

    DoEntFire("vsh_round_end*", "Trigger", "", 0, null, null);
    if (winner == TF_TEAM.MERCS)
        DoEntFire("vsh_mercs_win*", "Trigger", "", 0, null, null);
    else
        DoEntFire("vsh_hale_win*", "Trigger", "", 0, null, null);
    isRoundOver = true;

    //DisplayDamageBoard(winner);
}

function TF2M_EndGame()
{
    local roundWin = SpawnEntityFromTable("info_target",
    {
        targetname = "TF2M_ForceLastRoundFeedback"
    });
    local roundWin = SpawnEntityFromTable("game_round_win",
    {
        win_reason = "0",
        force_map_reset = "1", //not having
        TeamNum = winner,      //these 3 lines
        switch_teams = "0"     //causes the crash when trying to fire game_round_win
    });
    EntFireByHandle(roundWin, "RoundWin", "", 0, null, null);
}


/*function DisplayDamageBoard(winner)
{
    local hale = GetHale();
    local haleHP = IsValidPlayer(hale) ? IsPlayerDead(hale) ? 0 : hale.GetHealth() : "N/A";
    local text1 = "Damage dealt this round∶";
    local text2 = "Most damage dealt by∶";
    local i = 0;
    foreach (entry in GetDamageBoardSorted())
    {
        local player = entry[0];
        local damage = entry[1];
        if (!IsValidPlayer(player))
            continue;
        local name = GetPropString(player, "m_szNetname");
        local clazz = TF_CLASS_NAMES[player in starterClass ? starterClass[player] : 0];
        text1 += "\n"+damage+" - "+name+" ("+clazz+")";
        if (i++ < 3)
            text2 += "\n"+damage+" - "+name+" ("+clazz+")";
    }
    if (i == 0)
        text1 = text2 = (winner == TF_TEAM.MERCS ? "Hale bailed out" : "You're all losers!");
    Say(null, "Hale's health was: " + haleHP + "/" + haleStartHealth+"\n"+text1, false);

    text2+="\n\nScore∶\nHale  |  Mercs\n";

    local teamEntity = null;
    while (teamEntity = Entities.FindByClassname(teamEntity, "tf_team"))
    {
        if (GetPropInt(teamEntity, "m_iTeamNum") == TF_TEAM.HALE)
            text2+=GetPropInt(teamEntity, "m_iScore");
    }
    teamEntity = null;
    while (teamEntity = Entities.FindByClassname(teamEntity, "tf_team"))
    {
        if (GetPropInt(teamEntity, "m_iTeamNum") == TF_TEAM.MERCS)
            text2+="          " + GetPropInt(teamEntity, "m_iScore");
    }
    DoEntFire("game_text_victory_screen", "AddOutput", "message "+text2, 0, null, null);
    foreach (player in GetAllValidPlayers())
        ClientPrint(player, 3, text1);
}*/

/*function HijackVictoryPanel(params)
{
    params = params_ass;
    local tf_player_manager = Entities.FindByClassname(null, "tf_player_manager");

    local player_1 = "player_1" in params ? params.player_1 : 0
    local player_1_damage = GetPropIntArray(tf_player_manager, "m_iDamage", player_1);
    local player_1_healing = GetPropIntArray(tf_player_manager, "m_iHealing", player_1);

    local player_2 = "player_2" in params ? params.player_2 : 0
    local player_2_damage = GetPropIntArray(tf_player_manager, "m_iDamage", player_2);
    local player_2_healing = GetPropIntArray(tf_player_manager, "m_iHealing", player_2);

    local player_3 = "player_3" in params ? params.player_3 : 0
    local player_3_damage = GetPropIntArray(tf_player_manager, "m_iDamage", player_3);
    local player_3_healing = GetPropIntArray(tf_player_manager, "m_iHealing", player_3);

    SendGlobalGameEvent("arena_win_panel", {
        panel_style = params.panel_style,
        winning_team = params.winning_team,
        winreason = params.winreason,
        winning_team = params.winning_team,
        cappers = params.cappers,
        flagcaplimit = params.flagcaplimit,
        blue_score = params.blue_score,
        red_score = params.red_score,
        blue_score_prev = params.blue_score_prev,
        red_score_prev = params.red_score_prev,
        round_complete = params.round_complete,
        player_1 = player_1,
        player_1_damage = player_1_damage,
        player_1_healing = player_1_healing,
        //player_1_lifetime = player_1_lifetime,
        player_2 = player_2,
        player_2_damage = player_2_damage,
        player_2_healing = player_2_healing,
        //player_2_lifetime = player_2_lifetime,
        player_3 = player_3,
        player_3_damage = player_3_damage,
        player_3_healing = player_3_healing,
        //player_3_lifetime = player_3_lifetime,
    });
}*/

function DisplayDamageBoard(winner)
{
    local topPlayers = GetDamageBoardSorted();

    local tf_player_manager = Entities.FindByClassname(null, "tf_player_manager");

    local player_1 = topPlayers.len() > 0 ? topPlayers[0][0].entindex() : 0
    local player_1_damage = GetPropIntArray(tf_player_manager, "m_iDamage", player_1);
    local player_1_healing = GetPropIntArray(tf_player_manager, "m_iHealing", player_1);
    local player_1_lifetime = topPlayers.len() > 0 ? GetPropFloat(topPlayers[0][0], "m_flDeathTime") - roundStartTime : 0;
    if (player_1_lifetime < 0)
        player_1_lifetime = Time() - roundStartTime;

    local player_2 = topPlayers.len() > 1 ? topPlayers[1][0].entindex() : 0
    local player_2_damage = GetPropIntArray(tf_player_manager, "m_iDamage", player_2);
    local player_2_healing = GetPropIntArray(tf_player_manager, "m_iHealing", player_2);
    local player_2_lifetime = topPlayers.len() > 1 ? GetPropFloat(topPlayers[1][0], "m_flDeathTime") - roundStartTime : 0;
    if (player_2_lifetime < 0)
        player_2_lifetime = Time() - roundStartTime;
    local player_3 = topPlayers.len() > 2 ? topPlayers[2][0].entindex() : 0
    local player_3_damage = GetPropIntArray(tf_player_manager, "m_iDamage", player_3);
    local player_3_healing = GetPropIntArray(tf_player_manager, "m_iHealing", player_3);
    local player_3_lifetime = topPlayers.len() > 2 ? GetPropFloat(topPlayers[2][0], "m_flDeathTime") - roundStartTime : 0;
    if (player_3_lifetime < 0)
        player_3_lifetime = Time() - roundStartTime;

    local mercs_score = 0;
    local hale_score = 0;
    local teamEntity = null;
    while (teamEntity = Entities.FindByClassname(teamEntity, "tf_team"))
    {
        if (GetPropInt(teamEntity, "m_iTeamNum") == TF_TEAM.HALE)
            hale_score = GetPropInt(teamEntity, "m_iScore");
        else if (GetPropInt(teamEntity, "m_iTeamNum") == TF_TEAM.MERCS)
            mercs_score = GetPropInt(teamEntity, "m_iScore");
    }

    SendGlobalGameEvent("arena_win_panel", {
        panel_style = 0,
        winning_team = winner,
        winreason = 1,
        blue_score = hale_score,
        red_score = mercs_score,
        blue_score_prev = hale_score,
        red_score_prev = mercs_score,
        round_complete = 1,
        player_1 = player_1,
        player_1_damage = player_1_damage,
        player_1_healing = player_1_healing,
        player_1_lifetime = player_1_lifetime,
        player_2 = player_2,
        player_2_damage = player_2_damage,
        player_2_healing = player_2_healing,
        player_2_lifetime = player_2_lifetime,
        player_3 = player_3,
        player_3_damage = player_3_damage,
        player_3_healing = player_3_healing,
        player_3_lifetime = player_3_lifetime,
        flagcaplimit = 1
        cappers = "",
    });
}