//=========================================================================
//Copyright LizardOfOz.
//This is still beta and things might and will change over time.
//
//Check the following links for updates:
//   https://tf2maps.net/members/lizardofoz.39426/#resources
//   https://twitter.com/LizardOfOz_
//   https://youtube.com/LizardOfOz
//
//Credits:
//  LizardOfOz - the original VSH Plugin. VSH Vscript recreation.
//  Maxxy - Saxton Hale's model imitating Jungle Inferno SFM; Custom animations and promotional material.
//  Velly - VFX, animations scripting, technical assistance
//  JPRAS - Mayann Hale body reference, model feedback.
//  MegapiemanPHD - Saxton Hale and Gray Mann voice acting.
//  Yakibomb - give_tf_weapon script bundle (used for Hale's first-person hands model).
//=========================================================================

haleSlamAbility <- null;
haleChargeAbility <- null;
haleSaxtonPunchAbility <- null;
haleBraveJumpAbility <- null;
debuffResistanceAbility <- null;
haleAbilities <- [];

class HaleChange extends CharacterChange
{
    function CheckTeam() { return IsHale(player); }
    function ShouldBreakDamageLoop(damage_type) { return damage_type == 1 || damage_type == 2; }
}

class HaleAbility
{
    meter = 0;

    function Tick(hale, interval) { }
    function FrameTick(hale) { }
    function IsReady(hale) { return false; }
    function Perform(hale) { }
    function MeterValueAsString() { return null; }
}

IncludeScript("vssaxtonhale/hale/hale_abilities/brave_jump.nut");
IncludeScript("vssaxtonhale/hale/hale_abilities/charge.nut");
IncludeScript("vssaxtonhale/hale/hale_abilities/mighty_slam.nut");
IncludeScript("vssaxtonhale/hale/hale_abilities/saxton_punch.nut");
IncludeScript("vssaxtonhale/hale/hale_abilities/stun_breakout.nut");
IncludeScript("vssaxtonhale/hale/hale_abilities/debuff_resistance.nut");
IncludeScript("vssaxtonhale/hale/hale_abilities/wide_punch.nut");
IncludeScript("vssaxtonhale/hale/hale_abilities/received_damage_scaling.nut");

function TickHaleAbilities(interval)
{
    local hale = GetHale();
    if (IsNotHale(hale))
        return;

    foreach (ability in haleAbilities)
        ability.Tick(hale, interval);
}
AddListener("tick_only_valid", TickHaleAbilities, 0);

function FrameTickHaleAbilities(void)
{
    local hale = GetHale();
    if (IsNotHale(hale))
        return;

    foreach (ability in haleAbilities)
        ability.FrameTick(hale);
}
AddListener("tick_frame", FrameTickHaleAbilities, 0);

function CreateAoE(center, radius, applyDamageFunc, applyPushFunc)
{
    foreach(target in GetAliveMercs())
    {
        local deltaVector = target.GetCenter() - center;
        local distance = deltaVector.Norm();
        if (distance > radius)
            continue;

        applyPushFunc(target, deltaVector, distance);
        applyDamageFunc(target, deltaVector, distance);
    }

    local target = null;
    while (target = Entities.FindByClassname(target, "obj_*"))
    {
        local deltaVector = target.GetCenter() - center;
        local distance = deltaVector.Norm();
        if (distance > radius)
            continue;

        applyDamageFunc(target, deltaVector, distance);
    }
}

function ResetAbilities()
{
    haleSlamAbility = SlamHaleAbility();
    haleChargeAbility = ChargeHaleAbility();
    haleSaxtonPunchAbility = SaxtonPunchHaleAbility();
    haleBraveJumpAbility = BraveJumpHaleAbility();
    debuffResistanceAbility = DebuffResistanceAbility();
    haleAbilities = [haleSlamAbility, haleChargeAbility, haleSaxtonPunchAbility, haleBraveJumpAbility, debuffResistanceAbility];
}

::HalePlayViewModelAnim <- function(animation)
{
    local main_viewmodel = GetPropEntity(GetHale(), "m_hViewModel")
    local sequenceId = main_viewmodel.LookupSequence(animation);
    if (main_viewmodel.GetSequence() != sequenceId)
        main_viewmodel.SetSequence(sequenceId)
}