//=========================================================================
//Copyright LizardOfOz.
//This is still beta and things might and will change over time.
//
//Check the following links for updates:
//   https://tf2maps.net/members/lizardofoz.39426/#resources
//   https://twitter.com/LizardOfOz_
//   https://youtube.com/LizardOfOz
//
//Credits:
//  LizardOfOz - the original VSH Plugin. VSH Vscript recreation.
//  Maxxy - Saxton Hale's model imitating Jungle Inferno SFM; Custom animations and promotional material.
//  Velly - VFX, animations scripting, technical assistance
//  JPRAS - Mayann Hale body reference, model feedback.
//  MegapiemanPHD - Saxton Hale and Gray Mann voice acting.
//  Yakibomb - give_tf_weapon script bundle (used for Hale's first-person hands model).
//=========================================================================

class SaxtonPunchHaleAbility extends HaleAbility
{
    meter = -30;
    function IsReady(hale)
    {
        return meter == 0;
    }

    function MeterValueAsString()
    {
        if (meter == 0)
            return "Ready";
        else if (meter < 0)
            return format("%.1fs", -meter);
        else
            return format("%d%%", meter * 100);
    }

    function Tick(hale, interval)
    {
        if (meter < 0 && !vsh_vscript.IsRoundSetup())
        {
            meter += interval;
            if (meter >= 0)
            {
                meter = 0;
                EmitSoundOn("VSH.PunchReadyVO", hale);

                hale.AddCond(Constants.ETFCond.TF_COND_CRITBOOSTED);

                HalePlayViewModelAnim("vsh_megapunch_ready");

                SetPropInt(hale, "m_nForcedSkin", 4);

                //todo this deletes first-person too
                local wearable = null;
                while (wearable = Entities.FindByClassname(wearable, "tf_wearable"))
                    if (wearable.GetOwner() == hale)
                        wearable.Kill();
                local viewmodel = null;
                    while (viewmodel = Entities.FindByClassname(viewmodel, "tf_wearable_vm"))
                        if (viewmodel.GetOwner() == hale)
                            viewmodel.SetBodygroup(1,1);
                hale.CreateCustomWearable(null, SAXTON_HALE_GLOW_MODEL_PATH);
                hale.CreateCustomWearable(null, SAXTON_HALE_GLOVES2_MODEL_PATH);
            }
        }
    }

    function Perform(hale, victim)
    {
        if (!IsReady(hale))
            return;
        meter -= 30;

        local haleEyeVector = hale.EyeAngles().Forward();
        haleEyeVector.Norm();

        local viewmodel = null;
        while (viewmodel = Entities.FindByClassname(viewmodel, "tf_wearable_vm"))
            if (viewmodel.GetOwner() == hale)
                viewmodel.SetBodygroup(1,0);

        hale.RemoveCond(Constants.ETFCond.TF_COND_CRITBOOSTED);
        EmitSoundOn("TFPlayer.CritHit", hale);
        EmitSoundOn("VSH.PunchSFX", hale);
        if (RandomInt(1,3))
            EmitSoundOn("VSH.Punch", hale);
        DispatchParticleEffect("vsh_megapunch_shockwave", victim.EyePosition(), QAngle(0,hale.EyeAngles().Yaw(),0).Forward());
        ScreenShake(hale.GetCenter(), 10, 2.5, 1, 1000, 0, true);

        local wearable = null;
        while (wearable = Entities.FindByClassname(wearable, "tf_wearable"))
            if (wearable.GetOwner() == hale)
                wearable.Kill();
        hale.CreateCustomWearable(null, SAXTON_HALE_GLOW_MODEL_PATH);
        hale.CreateCustomWearable(null, SAXTON_HALE_GLOVES_MODEL_PATH);

        SetPropInt(hale, "m_nForcedSkin", 0);

        vsh_vscript.CreateAoE(hale.GetCenter(), 600,
            function (target, deltaVector, distance) {
                local dot = haleEyeVector.Dot(deltaVector);
                if (dot < 0.6)
                    return;
                //DispatchParticleEffect("mvm_pow_punch", target.EyePosition() + Vector(0,0,30), Vector(0,0,0));
                target.TakeDamageEx(
                    hale,
                    hale,
                    hale.GetActiveWeapon(),
                    deltaVector,
                    hale.GetOrigin(),
                    target.GetMaxHealth() * (0.7 - distance / 2000),
                    1);
            }
            function (target, deltaVector, distance) {
                local dot = haleEyeVector.Dot(deltaVector);
                if (dot < 0.6)
                    return;
                local pushForce = distance < 100 ? 10 : 10 / sqrt(distance);
                deltaVector.x = deltaVector.x * 1250 * pushForce;
                deltaVector.y = deltaVector.y * 1250 * pushForce;
                deltaVector.z = 750 * pushForce;
                target.Yeet(deltaVector);
            });
    }
}