//=========================================================================
//Copyright LizardOfOz.
//This is still beta and things might and will change over time.
//
//Check the following links for updates:
//   https://tf2maps.net/members/lizardofoz.39426/#resources
//   https://twitter.com/LizardOfOz_
//   https://youtube.com/LizardOfOz
//
//Credits:
//  LizardOfOz - the original VSH Plugin. VSH Vscript recreation.
//  Maxxy - Saxton Hale's model imitating Jungle Inferno SFM; Custom animations and promotional material.
//  Velly - VFX, animations scripting, technical assistance
//  JPRAS - Mayann Hale body reference, model feedback.
//  MegapiemanPHD - Saxton Hale and Gray Mann voice acting.
//  Yakibomb - give_tf_weapon script bundle (used for Hale's first-person hands model).
//=========================================================================

::characterChangesClasses <- [];
::characterChanges <- {};
class CharacterChange
{
    lastFireTime = 0;
    lastWeapon = null;
    player = null;

    function constructor()
    {
    }

    function TryApply(player)
    {
        if (!IsValidPlayer(player))
            return;
        this.player = player;
        if (!CanApply() || !CheckTeam())
            return;
        if (!(player in characterChanges))
            characterChanges[player] <- [];
        characterChanges[player].push(this);
        OnApply();
    }
    function DoTick(interval)
    {
        OnTick(interval);
    }
    function DoFrameTick()
    {
        OnFrameTick();
    }

    function CanApply() { return true; }
    function CheckTeam() { return IsNotHale(player); }
    function OnApply() { }
    function OnFrameTick() { }
    function OnTick(interval) { }
    function OnDamageDealt(params) { }
    function OnDamageTaken(params) { }
    function OnDeath(params) { }
    function OnHurtDealtEvent(victim, params) { }
}

IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/spy_backstab.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/demo_katana.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/demo_shield.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/demo_sword.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/demo_sticky_hooray.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/engineer_sentry.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/heavy_minigun.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/medic_uber_rate.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/scout_candy_cane.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/sniper_headshot.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/soldier_airstrike.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/single_class/soldier_market_gardener.nut");

IncludeScript("vssaxtonhale/mercs/merc_changes/all_class/last_mann_standing.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/all_class/melee_crits.nut");
IncludeScript("vssaxtonhale/mercs/merc_changes/all_class/melee_wall_climb.nut");

function CharacterChanges_Spawn(player)
{
    characterChanges[player] <- [];
    foreach (characterChange in characterChangesClasses)
        try
        {
            local newChange = characterChange();
            newChange.TryApply.call(newChange, player);
        }
        catch(e) { }
}
AddListener("spawn", CharacterChanges_Spawn, -1);

function CharacterChanges_Tick(interval)
{
    foreach (player in GetAlivePlayers())
        if (player in characterChanges)
            foreach (characterChange in characterChanges[player])
                try { characterChange.DoTick.call(characterChange, interval); }
                catch(e) { }
}
AddListener("tick_always", CharacterChanges_Tick, 2);

function CharacterChanges_FrameTick(void)
{
    foreach (player in GetAlivePlayers())
        if (player in characterChanges)
            foreach (characterChange in characterChanges[player])
                try { characterChange.DoFrameTick.call(characterChange); }
                catch(e) { }
}
AddListener("tick_frame", CharacterChanges_FrameTick, 2);

function CharacterChanges_Death(params)
{
    local player;
    if ((player = GetPlayerFromParams(params)) == null)
        return;
    foreach (characterChange in characterChanges[player])
        try { characterChange.OnDeath.call(characterChange, params); }
        catch(e) { }
}
AddListener("death", CharacterChanges_Death, 0);

function CharacterChanges_Damage(params)
{
    local victim = params.const_entity;
    local attacker = params.attacker;

    if (IsValidPlayer(attacker))
        foreach (characterChange in characterChanges[attacker])
            try { characterChange.OnDamageDealt.call(characterChange, params); }
            catch(e) { }

    if (IsValidPlayer(victim))
        foreach (characterChange in characterChanges[victim])
            try { characterChange.OnDamageTaken.call(characterChange, params); }
            catch(e) { }
}
AddListener("damage_hook", CharacterChanges_Damage, 0);

function CharacterChanges_Hurt(params)
{
    local attacker = params[0];
    local victim = params[1];
    local eventParams = params[2];

    if (IsValidPlayer(attacker))
        foreach (characterChange in characterChanges[attacker])
            try { characterChange.OnHurtDealtEvent.call(characterChange, victim, eventParams); }
            catch(e) { }
}
AddListener("player_hurt", CharacterChanges_Hurt, 0);